from django.core.mail import send_mail
import json

from django.shortcuts import render, get_object_or_404, redirect, reverse
from .models import Inbox, DevelopmentEmail, InboxEmail
from django.db.models import Count
from django.contrib import messages
from django.views.decorators.clickjacking import xframe_options_sameorigin


# Create your views here.

def inboxesView(request):
    # display all the inboxes
    inboxes = Inbox.objects.all().annotate(Count('inboxemail')).order_by('-created')
    return render(request, 'developmentEmailDashboard/inboxesView.html', context={'inboxes': inboxes})


def inboxView(request, email):
    # display all sent and received emails for specific inbox
    inbox = get_object_or_404(Inbox, email=email)
    emails = InboxEmail.objects.filter(inbox=inbox, is_sender=False).order_by('-created')
    sentEmails = InboxEmail.objects.filter(inbox=inbox, is_sender=True).order_by('-created')
    return render(request, 'developmentEmailDashboard/inboxView.html', context={'inbox': inbox,
                                                                                'emails': emails,
                                                                                'sentEmails': sentEmails})


@xframe_options_sameorigin
def emailView(request, email, id):
    # displays actual email in a iframe
    inbox = get_object_or_404(Inbox, email=email)
    inboxes = Inbox.objects.all()
    email = get_object_or_404(InboxEmail, inbox=inbox, id=id)
    email.read = True
    email.save()
    return render(request, 'developmentEmailDashboard/emailView.html',
                  context={'inbox': inbox, 'email': email, 'inboxes': inboxes})


def replyEmail(request, email, id):
    # this email-backend does not support any replying to capability, so it
    # sends email with same subject to reply_to field

    inbox = get_object_or_404(Inbox, email=email)
    inboxemail = get_object_or_404(InboxEmail, inbox=inbox, id=id)

    toemails = []
    for em in json.loads(request.POST.get('reply_to')):
        toemails.append(em['tag'])
    send_mail(
        inboxemail.email.subject,
        request.POST.get('content', ''),
        request.POST.get('from_address', ''),
        toemails,
        fail_silently=False,
    )
    messages.info(request, f'sent reply to {toemails}')
    return redirect(reverse('developmentEmailDashboard:email_view', args=(email, id)))


def deleteEmail(request, email, id):
    # deletes the email for specified inbox if no other inboxes have this email
    # will delete models.DevelopmentEmail instance of it
    inbox = get_object_or_404(Inbox, email=email)
    inboxemail = get_object_or_404(InboxEmail, inbox=inbox, id=id)
    inboxemail.delete()
    messages.info(request, f'deleted email')
    return redirect(reverse('developmentEmailDashboard:inbox_view', args=(email,)))


def sendEmail(request, email):
    # sends a new email to email address
    toemails = []
    for em in json.loads(request.POST.get('to')):
        toemails.append(em['tag'])

    send_mail(
        request.POST.get('subject', ''),
        request.POST.get('content', ''),
        email,
        toemails,
        fail_silently=False,
    )
    messages.info(request, f'sent email to {toemails}')
    return redirect(reverse('developmentEmailDashboard:inbox_view', args=(email,)))


def deleteInbox(request, email):
    # deletes inbox with all of its emails
    inbox = get_object_or_404(Inbox, email=email)
    inbox.delete()
    messages.info(request, f'deleted Inbox "{email}"')
    return redirect(reverse('developmentEmailDashboard:inboxes_view'))


def createInboxView(request, email):
    # trys to create a new email and redirects to that
    # if the email inbox already exists redirects you to home page and sends message that the email already exists
    try:
        Inbox.objects.create(email=email)
        messages.success(request, f'created  {email} Inbox')
        return redirect(reverse('developmentEmailDashboard:inbox_view', args=(email,)))
    except:
        messages.error(request, f'cannot create Inbox with email "{email}" because it already exists')
        return redirect(reverse('developmentEmailDashboard:inboxes_view'))



# Generated by Django 4.0 on 2021-12-24 00:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('developmentEmailDashboard', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='developmentemail',
            name='email_type',
            field=models.CharField(default='text/plain', max_length=20),
            preserve_default=False,
        ),
    ]



from django.apps import AppConfig


class DevelopmentemaildashboardConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'developmentEmailDashboard'

    def ready(self):
        # setup signals for deleting a DevelopmentEmail when no InboxEmail objects are attached to it
        from . import signals



from django.urls import path
from . import views

app_name = 'developmentEmailDashboard'

urlpatterns = [
    path('', views.inboxesView, name='inboxes_view'),
    path('create/<str:email>/', views.createInboxView, name='create_inbox_view'),
    path('<str:email>/', views.inboxView, name='inbox_view'),
    path('<str:email>/<int:id>/', views.emailView, name='email_view'),
    path('<str:email>/<int:id>/delete/', views.deleteEmail, name='delete_email'),
    path('<str:email>/<int:id>/sendemail/', views.replyEmail, name='reply_view'),
    path('<str:email>/sendemail/', views.sendEmail, name='send_email_view'),
    path('<str:email>/delete/', views.deleteInbox, name='delete_inbox_view'),
]


# from django.contrib import admin
# from .models import DevelopmentEmail,Inbox,InboxEmail
# # Register your models here.
# admin.site.register(DevelopmentEmail)
# admin.site.register(Inbox)
# admin.site.register(InboxEmail)


# Generated by Django 4.0 on 2021-12-22 22:48

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='DevelopmentEmail',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('subject', models.CharField(max_length=200)),
                ('body', models.TextField()),
                ('from_email', models.EmailField(max_length=254)),
                ('extra_headers', models.JSONField()),
                ('to', models.JSONField()),
                ('bcc', models.JSONField()),
                ('cc', models.JSONField()),
                ('reply_to', models.JSONField()),
                ('created', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'ordering': ['-created'],
            },
        ),
        migrations.CreateModel(
            name='Inbox',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('email', models.EmailField(max_length=254, unique=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'ordering': ['-created'],
            },
        ),
        migrations.CreateModel(
            name='InboxEmail',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('read', models.BooleanField(default=False)),
                ('is_sender', models.BooleanField(default=False)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('email', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='developmentEmailDashboard.developmentemail')),
                ('inbox', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='developmentEmailDashboard.inbox')),
            ],
            options={
                'ordering': ['-created'],
            },
        ),
    ]



from django.db.models.signals import pre_save, post_delete
from django.dispatch import receiver
from .models import InboxEmail


@receiver(post_delete, sender=InboxEmail)
def delete_development_email_if_nothing_attached(sender, instance, using, **kwargs):
    # get the models.DevelopmentEmail from the models.InboxEmail
    try:
        developmentEmail = instance.email

        # get all the InboxEmails associated with the parent email (models.DevelopmentEmail)
        emails = InboxEmail.objects.filter(email=developmentEmail)

        # only delete the DevelopmentEmail object if no InboxEmails refer to it
        if not emails.exists():

            developmentEmail.delete()

    except Exception as e:
        print(e)



from django.core.mail.backends.base import BaseEmailBackend
from .models import DevelopmentEmail, Inbox, InboxEmail
from django.conf import settings
import webbrowser
notification_callback_ids = {}


class developmentEmailBackend(BaseEmailBackend):
    def send_messages(self, email_messages):
        for email in email_messages:
            devemail = DevelopmentEmail.objects.create(subject=email.subject, body=email.body,
                                                       email_type=email.content_subtype,
                                                       from_email=email.from_email, extra_headers=email.extra_headers,
                                                       to=email.to, bcc=email.bcc, cc=email.cc, reply_to=email.reply_to)

            # add email to senders outbox
            inbox, created = Inbox.objects.get_or_create(email=email.from_email)
            InboxEmail.objects.create(inbox=inbox, email=devemail, is_sender=True,read=True)

            # add email to to inbox
            for email_address in email.to:
                self.send_email(devemail, email_address)

            # add email to bcc inbox
            for email_address in email.bcc:
                self.send_email(devemail, email_address)

            # add email to cc inbox
            for email_address in email.cc:
                self.send_email(devemail, email_address)

        return len(email_messages)

    def send_email(self, email, emailAddress):
        inbox, created = Inbox.objects.get_or_create(email=emailAddress)
        InboxEmail.objects.create(inbox=inbox, email=email)

        # send a windows notification if send email notification is set to true
        try:
            if settings.DEVELOPMENT_EMAIL_DASHBOARD_SEND_EMAIL_NOTIFICATION:
                send_notification(email, inbox)
        except AttributeError:
            pass


def notificationClickHandler(notification_id):
    url = notification_callback_ids[notification_id]
    webbrowser.open(f'http://localhost:8000{url}')



def send_notification(email, inbox):
    try:
        import zroya

        # send notification
        zroya.init("django-developmentEmailDashboard", "dragoncommits",
                   'django-developmentEmailDashboard', 'django-developmentEmailDashboard', '1')
        t = zroya.Template(zroya.TemplateType.Text2)

        t.setFirstLine(f'"{inbox.email}" received a new email')
        t.setSecondLine(email.subject)
        id = zroya.show(t, on_click=notificationClickHandler)
        notification_callback_ids[id] = inbox.get_absolute_url()

    except ModuleNotFoundError:
        print(
            'to show notifications about receiving emails you must install zroya first install zroya | pip install zroya')

    except Exception:
        pass



from django.db import models
from django.urls import reverse, NoReverseMatch, get_script_prefix
from django.utils.encoding import iri_to_uri

# model for each email account with the email address
class Inbox(models.Model):
    email = models.EmailField(unique=True)
    created = models.DateTimeField(auto_now_add=True)

    class Meta:
        ordering = ['-created']

    def __str__(self):
        return self.email

    def get_unread_email_number(self):

        return InboxEmail.objects.filter(inbox=self,read=False).count()

    def get_absolute_url(self):
        try:
            return reverse('developmentEmailDashboard:inbox_view', kwargs={'email': self.email})
        except NoReverseMatch:
            pass
        # Handle script prefix manually because we bypass reverse()
        return iri_to_uri(get_script_prefix().rstrip('/') + self.url)

# model for the actual email message
class DevelopmentEmail(models.Model):
    subject = models.CharField(max_length=200)
    body = models.TextField()
    from_email=models.EmailField()
    extra_headers = models.JSONField()
    to = models.JSONField()
    bcc = models.JSONField()
    cc = models.JSONField()
    reply_to = models.JSONField()
    email_type = models.CharField(max_length=20)

    created = models.DateTimeField(auto_now_add=True)

    class Meta:
        ordering = ['-created']

    def __str__(self):
        return f'from: {self.from_email} |  {self.subject[:20]}...'


# model for the unique email the user deletes/reads/sends etc
class InboxEmail(models.Model):
    inbox = models.ForeignKey(Inbox,on_delete=models.CASCADE)
    email = models.ForeignKey(DevelopmentEmail,on_delete=models.CASCADE)

    read = models.BooleanField(default=False)
    is_sender = models.BooleanField(default=False)
    created = models.DateTimeField(auto_now_add=True)
    class Meta:
        ordering = ['-created']





from django.test import TestCase

# I don't know how to test sending emails because django switches the email backend to
# django.core.mail.backends.locmem.EmailBackend for testing
# if someone would like to contribute tests that run only in development that would be greatly appreciated
